/*
	This file is part of FireBible, a bible study tool for Firefox.
	Copyright (C) 2008,2009 Brian Fernandes <brian@thegoan.com>

    FireBible is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/

function FireBible() {
	this.cCount = 0;
	this.name = "chrome://firebible";
	this.bControls = new Array();	
	this.windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1']
	                                        .getService(Components.interfaces.nsIWindowMediator);
	this.promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
	                                        .getService(Components.interfaces.nsIPromptService);
    this.observerService = Components.classes["@mozilla.org/observer-service;1"]
                                      .getService(Components.interfaces.nsIObserverService);
    this.annotSvc = Components.classes["@mozilla.org/browser/annotation-service;1"]
                                      .getService(Components.interfaces.nsIAnnotationService);
    this.observerService.addObserver(this, this.name, false);	
	this.log("Firebible Version 1.0.0");
	this.verseEx = new RegExp("^(\\d*\\D+)((?:[^a-eg-z\\s]|\\s(?!f))*)", "i");
	this.registerProtocol();
};

FireBible.prototype.getJSword = function() {
	this.jsword = this.pHandler.getJSword();
	if (!this.jsword) {
		this.jsword = new JSword();
		this.pHandler.setJSword(this.jsword);
		this.pHandler2.setJSword(this.jsword);
		this.jsword.setComponentLocation(this.pHandler.getLocation().parent.path);
	}
	return this.jsword;
};

FireBible.prototype.initializeMenus = function() {
	var prefs = this.getJSword().fbPrefs;
	
	this.initMenuItem(prefs, prefs.VERSE_SEPARATE_LINE);
	this.initMenuItem(prefs, prefs.VERSE_SUPERSCRIPT);
	this.initMenuItem(prefs, prefs.SHOW_HEADINGS);
	this.initMenuItem(prefs, prefs.SHOW_NOTES);
	
	var verseFormat = prefs.getIntPref(prefs.VERSE_FORMAT);
	var menuItem = document.getElementById(prefs.VERSE_FORMAT + "." + verseFormat);
	menuItem.setAttribute("checked", true);
};

FireBible.prototype.initMenuItem = function(prefs, id) {
	var menuItem = document.getElementById(id);
	var checked = prefs.getBoolPref(id);
	menuItem.setAttribute("checked", checked);
};

FireBible.prototype.toggleMenuItem = function(menuItem) {
	var checked = menuItem.getAttribute("checked");	
	checked = (!checked || checked == "false") ? false : true;
	this.jsword.fbPrefs.setBoolPref(menuItem.id, checked);
	this.reload();	
};

FireBible.prototype.setVerseFormat = function(format) {
	this.jsword.fbPrefs.setIntPref(this.jsword.fbPrefs.VERSE_FORMAT, format);
	this.reload();
};

FireBible.prototype.reload = function() {
	if (gBrowser.selectedBrowser.currentURI != null) {
		var uri = gBrowser.selectedBrowser.currentURI.spec;
		if (uri.indexOf("bible://") == 0 || uri.indexOf("sword://") == 0) {
			gBrowser.selectedBrowser.reload();
		}
	}	
};

FireBible.prototype.initFromProtocol = function(silent) {
	this.bibleControl.initBibles(silent);
	//this.bibleControl.selectFirstBible();
};

FireBible.prototype.initBibleList = function(forceReset) {
	var bibles = this.getJSword().getBibleList(forceReset, false);	
	this.bibleControl.bibles = bibles;
	return bibles != null;
};

FireBible.prototype.initBookList = function() {
	this.bibleControl.books = this.getJSword().getBookList();
};

FireBible.prototype.onLoad = function() {
	this.toolbarItem = document.getElementById("firebible-toolbarItem");
	this.manageBookmarks();
	try {
		this.setupUbiquity();
	} catch (e) {
		this.log("Error installing Ubiquity support. Please ensure you have Ubiquity 0.5 or higher.");
	}
};

FireBible.prototype.setupUbiquity = function() {
	var url = "chrome://firebible/content/ubiquity.js";		
	// link to the ubiquity setup module.
	var jsm = {}; 
	Components.utils.import("resource://ubiquity/modules/setup.js", jsm);	
	if (jsm.UbiquitySetup) {
		jsm.UbiquitySetup.preload(function() {    	
			try {
				jsm.UbiquitySetup.createServices().feedManager.addSubscribedFeed( {
					url: url,
					sourceUrl: url,
					canAutoUpdate: false,
					isBuiltIn: true
				});
			} catch (e) {
				//do nothing
			}
		});
	}
};

FireBible.prototype.registerProtocol = function() {
	var bibleComponent = Components.classes["@mozilla.org/network/protocol;1?name=bible"];
	this.pHandler = bibleComponent.getService().wrappedJSObject;
	bibleComponent = Components.classes["@mozilla.org/network/protocol;1?name=sword"];
	this.pHandler2 = bibleComponent.getService().wrappedJSObject;
};

FireBible.prototype.manageBookmarks = function() {
	var bmsvc = Components.classes["@mozilla.org/browser/nav-bookmarks-service;1"];
	if (!bmsvc) return
	bmsvc = bmsvc.getService(Components.interfaces.nsINavBookmarksService);
	
	var ios = Components.classes["@mozilla.org/network/io-service;1"]
	                             .getService(Components.interfaces.nsIIOService);	
	var tFolder = bmsvc.bookmarksMenuFolder;	
	var history = bmsvc.getChildFolder(tFolder, "FireBible History");
	var bookmarks = bmsvc.getChildFolder(tFolder, "FireBible Bookmarks");
	var index;
	if (history) {
		index = bmsvc.getItemIndex(history) + 1;
	} else if (bookmarks) {
		index = bmsvc.getItemIndex(bookmarks);
	} else {
		index = bmsvc.DEFAULT_INDEX;
	}
	
	if (!history) {
		//create history queries
		history = bmsvc.createFolder(tFolder, "FireBible History", index);
		this.insertBookmark(bmsvc, ios, "sort=4&maxResults=40&!annotation=firebible/bookCategory/dictionary", "Last 40", history);
		this.insertBookmark(bmsvc, ios, "beginTimeRef=2&beginTime=-7200000000&endTimeRef=2&endTime=0&sort=4", "Last 2 hours", history);
		this.insertBookmark(bmsvc, ios, "sort=8&maxResults=20", "Most Visited", history);
		bmsvc.insertSeparator(history, bmsvc.DEFAULT_INDEX);
		this.insertBookmark(bmsvc, ios, "sort=4&maxResults=40&annotation=firebible/bookCategory/bible", "Bibles/Commentaries", history);
		this.insertBookmark(bmsvc, ios, "sort=4&maxResults=40&annotation=firebible/bookCategory/genbook", "Gen Books/Daily Devotionals/Essays", history);
		this.insertBookmark(bmsvc, ios, "sort=4&maxResults=40&annotation=firebible/bookCategory/maps", "Maps/Images", history);
		this.insertBookmark(bmsvc, ios, "sort=4&maxResults=40&annotation=firebible/bookCategory/dictionary", "Dictionaries/Glossaries", history);
		bmsvc.insertSeparator(history, bmsvc.DEFAULT_INDEX);
		this.insertBookmark(bmsvc, ios, "beginTimeRef=1&beginTime=0&sort=1", "Today", history);
		this.insertBookmark(bmsvc, ios, "beginTimeRef=1&beginTime=-86400000000&endTimeRef=1&endTime=0&sort=1", "Yesterday", history);
		this.insertBookmark(bmsvc, ios, "beginTimeRef=1&beginTime=-172800000000&endTimeRef=1&endTime=-86400000000&sort=1", "2 days ago", history);
		this.insertBookmark(bmsvc, ios, "beginTimeRef=1&beginTime=-259200000000&endTimeRef=1&endTime=-172800000000&sort=1", "3 days ago", history);
		this.insertBookmark(bmsvc, ios, "beginTimeRef=1&beginTime=-345600000000&endTimeRef=1&endTime=-259200000000&sort=1", "4 days ago", history);
		this.insertBookmark(bmsvc, ios, "beginTimeRef=1&beginTime=-432000000000&endTimeRef=1&endTime=-345600000000&sort=1", "5 days ago", history);
		this.insertBookmark(bmsvc, ios, "beginTimeRef=1&beginTime=-518400000000&endTimeRef=1&endTime=-432000000000&sort=1", "6 days ago", history);
		this.insertBookmark(bmsvc, ios, "&endTimeRef=1&endTime=-518400000000&sort=4&maxResults=40", "Older than 6 days", history);
	}	
	
	if (!bookmarks) {
		//create bookmark queries
		bookmarks = bmsvc.createFolder(tFolder, "FireBible Bookmarks", index);
		this.insertBookmark(bmsvc, ios, "queryType=1&sort=1", "All", bookmarks);
		bmsvc.insertSeparator(bookmarks, bmsvc.DEFAULT_INDEX);
		this.insertBookmark(bmsvc, ios, "annotation=firebible/bookCategory/bible&queryType=1&sort=1", "Bibles/Commentaries", bookmarks);
		this.insertBookmark(bmsvc, ios, "annotation=firebible/bookCategory/genbook&queryType=1&sort=1", "Gen Books/Daily Devotionals/Essays", bookmarks);
		this.insertBookmark(bmsvc, ios, "annotation=firebible/bookCategory/maps&queryType=1&sort=1", "Maps/Images", bookmarks);
		this.insertBookmark(bmsvc, ios, "annotation=firebible/bookCategory/dictionary&queryType=1&sort=1", "Dictionaries/Glossaries", bookmarks);
		bmsvc.insertSeparator(bookmarks, bmsvc.DEFAULT_INDEX);
		var bQuery = "folder=BOOKMARKS_MENU&folder=UNFILED_BOOKMARKS&folder=TOOLBAR&maxResults=20&queryType=1&";
		this.insertBookmark(bmsvc, ios, bQuery + "sort=12", "Recently Bookmarked", bookmarks);
		this.insertBookmark(bmsvc, ios, bQuery + "sort=4", "Recently Visited", bookmarks);
		this.insertBookmark(bmsvc, ios, bQuery + "sort=8", "Most Visited", bookmarks);
	}
	
	this.fbFolderIds = [history, bookmarks];
	this.bibleControl.initFBFolder(this.fbFolderIds);
};

FireBible.prototype.insertBookmark = function(bmsvc, ios, place, name, folder) {
	var uri = ios.newURI("place:terms=bible://&OR&terms=sword://&" + place, null, null);
	bmsvc.insertBookmark(folder, uri, bmsvc.DEFAULT_INDEX, name);
};

FireBible.prototype.initUtils = function(silent) {
	var js = this.getJSword();
	var inited = false;
	if (!js.initializing) {
		js.initializing = true;
		try {	    
			inited = js.initUtils(silent);
		} catch (e) {
			e = js.getRootException(e);
			this.log(e);
			this.printStackTrace(e);
		}
		js.initializing = false;
	} else if (!silent) {
		this.promptService.alert(window, "Info", "JSword is currently being initialized, please try again in a few seconds.");		
	}
	return inited;
};

FireBible.prototype.register = function(element) {
	this.bibleControl = element;
	if (this.fbFolderIds) {
		this.bibleControl.initFBFolder(this.fbFolderIds);
	}
};

FireBible.prototype.onUnload = function() {
};

FireBible.prototype.getGBrowser = function() {
	return gBrowser;
};

FireBible.prototype.log = function(message) {
	if (this.jsword && !this.jsword.fbPrefs.getBoolPref(this.jsword.fbPrefs.DEBUG)) return;
	if (this._logService == null) {
	        this._logService = Components.classes['@mozilla.org/consoleservice;1'].getService();
	        this._logService.QueryInterface(Components.interfaces.nsIConsoleService);
	}
	
	this._logService.logStringMessage("FireBible: " + message);
};

FireBible.prototype.printStackTrace = function(e) {
	var traces = e.getStackTrace();
	for (var i = 0; i < traces.length; i++) {
		this.log(traces[i].toString());
	}
};

FireBible.prototype.openPrefs = function() {
	if (this.windowManager.getMostRecentWindow("firebible:preferences") == null) {
		window.openDialog('chrome://firebible/content/preferences.xul', '_blank');
	}
};

FireBible.prototype.loadErrorHTML = function(errorMsg) {
	this.jsword.error = errorMsg;
	//var url = "sword://status/";
	var url = "chrome://firebible/content/status.html";
	gBrowser.loadURI(url);		
};

FireBible.prototype.loadHTML = function(bible, book, longName, chapter) {	
	var url = "bible://" + book;
	if (chapter != "ALL") {
		url += "." + chapter;// + ".1-ff";
	}
	var ios = Components.classes["@mozilla.org/network/io-service;1"]
	                             .getService(Components.interfaces.nsIIOService);
	var uri = ios.newURI(url, null, null);
	gBrowser.loadURI(url);	
	//gBrowser.setIcon(gBrowser.mCurrentTab, "chrome://firebible/skin/bible16.png");

	/*var uri2 = ios.newURI("chrome://firebible/skin/bible16.png", null, null);
	var browser = gBrowser.getBrowserForTab(gBrowser.mCurrentTab);
	browser.mIconURL = uri2;
	var faviconService = Components.classes["@mozilla.org/browser/favicon-service;1"]
	                                        .getService(Components.interfaces.nsIFaviconService);
	faviconService.setAndLoadFaviconForPage(uri, uri2, true);
	gBrowser.updateIcon(gBrowser.mCurrentTab);*/
	
	//this.changeIcon(gBrowser.mCurrentTab, "chrome://firebible/skin/bible16.png");
};

FireBible.prototype.changeIcon = function(aTab, aURI) {
    var browser = gBrowser.getBrowserForTab(aTab);
    browser.mIconURL = aURI;

    if (aURI) {
      if (!(aURI instanceof Components.interfaces.nsIURI)) {
        var ios = Components.classes["@mozilla.org/network/io-service;1"]
                            .getService(Components.interfaces.nsIIOService);
        aURI = ios.newURI(aURI, null, null);
      }
      gBrowser.mFaviconService.setAndLoadFaviconForPage(browser.currentURI,
                                                    aURI, false);
    }

    gBrowser.updateIcon(aTab);

    for (var i = 0; i < gBrowser.mProgressListeners.length; i++) {
      var p = gBrowser.mProgressListeners[i];
      if ('onLinkIconAvailable' in p)
        try {
        	this.log("once");
          p.onLinkIconAvailable(browser);
        } catch (e) {
          // don't inhibit other listeners
          Components.utils.reportError(e);
        }
    }
}

FireBible.prototype.load = function(uri) {
	var jSword = this.getJSword();
	var bible = this.getBible();
	var uriS = unescape(uri.host + uri.path);
	var bookKeyString = jSword.utilKlass.getBookFromKey(bible, uriS);
	var bClass = jSword.utilKlass.getBookClassCategory(bookKeyString[0]);		
	this.annotSvc.setPageAnnotation(uri, "firebible/bookCategory/" + bClass[1], bClass[0], 0, this.annotSvc.EXPIRE_WITH_HISTORY);
	if (bClass[0] == "bible" || bClass[0] == "questionable") {
		//load commentary
		this.lastBibleURI = bookKeyString[1];
		var sidebarWindow = document.getElementById("sidebar").contentWindow;
		if (sidebarWindow.location.href == "chrome://firebible/content/sidebar.xul") {
			sidebarWindow.fbItemList.loadCommentary();
		}
	}
	return jSword.getHTML(bible, uriS, 0);
};

FireBible.prototype.getBible = function() {
	var bible = this.bibleControl.bible;
	if (bible == null) {
		bible = this.getJSword().getBible();
	}
	return bible;
};

FireBible.prototype.isIndexed = function(bible) {
	return this.jsword.utilKlass.isIndexed(bible);
};

FireBible.prototype.search = function(bible, search) {
	this.log("searching for " + search); 
	try {
		this.jsword.pageTitle = "Search results for " + search;
		var key = this.jsword.utilKlass.find(bible, search, this.jsword.cl);
		if (!key || key == "") {
			this.loadErrorHTML("No results found");
			return;						
		}
		var uri = "bible://" + key;
		gBrowser.loadURI(uri);
	} catch (e) {
		e = this.jsword.getRootException(e);
		this.promptService.alert(window, "Error", "Error occurred while searching:\n" + e);
		this.log(e);
		this.printStackTrace(e);
	}
};

FireBible.prototype.index = function(bible, stack, bibleControl) {
	this.bibleControl = bibleControl;
	this.progressMeter = stack.firstChild;
	this.indexLabel = stack.lastChild.firstChild;
	this.progressMeter.value = 0;
	this.indexLabel.value = "";
	this.iManager = this.jsword.utilKlass.index(bible);
	this.indexInterval = setInterval("fireBible.checkIndexStatus();", 250);
};

FireBible.prototype.notify = function(message) {
	this.observerService.notifyObservers(this, this.name, message);
};

FireBible.prototype.observe = function(subject, topic, data) {
	if (data == this.getJSword().fbPrefs.MODULE_DIRS) {
		this.log("module directory list modified");
		this.initBibleList(true);
	} else if (data == this.getJSword().fbPrefs.INCLUDE_COMM) {
		this.initBibleList(false);
	}
};

FireBible.prototype.checkIndexStatus = function() {
	var details = this.iManager.getIndexStatus();
	this.progressMeter.value = details[1];	//work completed
	this.indexLabel.value = details[2];		//current work section
	var fin;
	if (details[0].getClass) {
		fin = details[0].booleanValue();
	} else {
		fin = details[0];
	}
	if (fin) {
		clearInterval(this.indexInterval);
		this.bibleControl.resetSearch();
		if (details[3] == null && this.isIndexed(this.iManager.getBook())) {
			this.bibleControl.doSearch(null, true);
		} else {
			this.promptService.alert(window, "Error", "The following error occurred while indexing this bible:\n" + this.jsword.getRootException(details[3]));
		}
	}
};

FireBible.prototype.suggestBooks = function(text) {	
	var books = new Array();
	//get the book name from the text
	var temp = this.verseEx.exec(text);
	if (temp == null || temp.length < 2) return books;
	text = this.trim(temp[1]);	
	var jsword = this.getJSword();
	if (!jsword || !jsword.utilsInitialized) {
		return books;
	}
	var booklist = this.getJSword().getBookList();
	for (var i = 0; i < booklist.length; i++) {		
		var name = booklist[i].name;
		if (name.match(text, "i")) {
			var mStart = (name.match("^" + text, "i") != null);
			books.push([name, booklist[i].sName, mStart]);
		}
	}
	
	if (books.length == 0) {
		return books;
	}
	
	if (temp.length > 2) {
		var verses = this.trim(temp[2]);
		if (verses.length > 0) {
			var finalBooks = new Array();
			var bible = this.jsword.getBible();
			for (var i = 0; i < books.length; i++) {
				var key = books[i][1] + "." + verses;
				var valid = this.jsword.utilKlass.validReference(key);
				if (valid) {
					finalBooks.push([books[i][0] + " " + verses, books[i][1], books[i][2]]);
				}								
			}
			if (finalBooks.length == 0) {
				return null;
			} else {
				return finalBooks;
			}
		}
	}
	return books;	
};

FireBible.prototype.suggestBibles = function(text) {
	var bibles = new Array();
	var jsword = this.getJSword();
	if (!jsword || !jsword.utilsInitialized) {
		return bibles;
	}
	for (var i = 0; i < jsword.bibleList.size(); i++) {
		var bible = jsword.bibleList.get(i);
		if (bible[0].match(text, "i") || bible[2].match(text, "i")) {
			var mStart = (bible[0].match("^" + text, "i") || bible[2].match("^" + text, "i"));
			bibles.push([bible[0], null, mStart]);
		}
	}	
	return bibles;
}

FireBible.prototype.trim = function(text) {
	return text.replace(/^\s+|\s+$/g, '');
};

var fireBible = new FireBible();

window.addEventListener("load", function() { fireBible.onLoad(); }, false);
window.addEventListener("unload", function() { fireBible.onUnload(); }, false);