/*
	This file is part of FireBible, a bible study tool for Firefox.
	Copyright (C) 2008,2009 Brian Fernandes <brian@thegoan.com>

    FireBible is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/
var mainWindow;
var fb;

function init_sidebar() {
	mainWindow = window.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                   .getInterface(Components.interfaces.nsIWebNavigation)
                   .QueryInterface(Components.interfaces.nsIDocShellTreeItem)
                   .rootTreeItem
                   .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                   .getInterface(Components.interfaces.nsIDOMWindow);
	fb = mainWindow.fireBible;
	fbItemList.init();
	fbTreeView.init();
	gNavigatorBundle = document.getElementById('bundle_browser');
}

function bookmarkThisPage(browser) {//sets the "load in sidebar" flag	
	var uri = browser.currentURI;
	var itemId = PlacesUtils.getMostRecentBookmarkForURI(uri);
	var existing = (itemId != -1);
	window.top.PlacesCommandHook.bookmarkPage(browser, PlacesUtils.bookmarksMenuFolderId, true);
	if (!existing) { //only if bookmark did not exist at the start
		var newItemId = PlacesUtils.getMostRecentBookmarkForURI(uri);
		if (newItemId != -1) { //bookmark was created (user could have clicked cancel)
			var txn = PlacesUIUtils.ptm.setLoadInSidebar(newItemId, true);
			PlacesUIUtils.ptm.doTransaction(txn);
		}	
	}
}

var fbItemList = {
	
	init : function() {
		this.listbox = document.getElementById("firebible-itemlist");
		if (fb.initUtils()) {
			this.jSword = fb.getJSword();
			this.fillItemList();
		} else {
			mainWindow.toggleSidebar('viewFireBibleSidebar');
		}
	},
	
	fillItemList : function() {
		this.modules = new Array();
		var bibleList = this.jSword.getSidebarModules();
		for (var i = 0; i < bibleList.size(); i++) {
			var bible = bibleList.get(i);
			this.modules[i] = bible;
			var listItem = this.listbox.appendItem(bible.getName());
			var bClass = this.jSword.utilKlass.getBookClass(bible);			
			listItem.setAttribute("class","listitem-iconic " + bClass);
		}
	},
	
	selectionChanged : function() {
		var module = this.modules[this.listbox.currentIndex];
		fbTreeView.update(module, this.jSword);
	},
	
	loadCommentary: function() {
		var module = this.modules[this.listbox.currentIndex];
		if (!this.jSword.utilKlass.isCommentary(module)) return;
		var uri = "sword://" + module.getInitials() + "/" + fb.lastBibleURI;
		var mini = document.getElementById("firebible-mini");
		mini.loadURI(uri);
		fbTreeView.expandCommentaryView(true);
	}
};

var fbTreeView = {
	
	init: function() {		
		this.tree = document.getElementById("firebible-tree");
		this.mini = document.getElementById("firebible-mini");
		this.splitter = document.getElementById("firebible-splitter2"); 
		this.tree.view = this;
	},
	
	visibleData : [],
	
	update : function(module, jSword) {
		this.module = module;
		this.jSword = jSword;
		var removed = this.visibleData.length;
		this.visibleData = new Array();
		if (jSword.utilKlass.isCommentary(module)) {
			if (fb.lastBibleURI != null) {
				fbItemList.loadCommentary();
			}
			this.treeBox.rowCountChanged(0, -removed);			
		} else {
			var prefIdx = this.addChildren(module.getGlobalKeyList(), 0, 0, removed, 
							jSword.utilKlass.isListBook(module), this.module);
			this.expandCommentaryView(false);			
			if (prefIdx && prefIdx != -1) {
				this.tree.view.selection.select(prefIdx);
			}
		}
	},
	
	expandCommentaryView : function(expand) {
		this.splitter.setAttribute("state", (expand ? "collapsed" : "open"));		
	},
	
	addChildren : function(pKey, index, level, removed, isList, module) {		
		var keys = this.jSword.utilKlass.getKeyChildren(pKey, module);
		var nChildren;
		var temp = new Array();
		if (isList) {
			var parts = keys[0].split("\n");
			nChildren = parts.length;
			for (var i = 0; i < nChildren; i++) {
				var jItem = parts[i];
				temp[i] = [null, level, jItem, false, false];
			}			
		} else {
			nChildren = keys.length;
			for (var i = 0; i < nChildren; i++) {
				var jItem = keys[i];
				temp[i] = [jItem[0], level, jItem[1], jItem[2], false]; //value, level, text, isContainer, isContainerOpen
			}
		}
		
		if (index == 0) {
			this.visibleData = temp;
		} else {
			for (var i = 0; i < nChildren; i++) {
				this.visibleData.splice(index + i, 0, temp[i]);
			}
		}
		this.treeBox.rowCountChanged(index, nChildren - removed);
		if (isList) {
			return keys[1];
		}
	},

	treeBox: null,
  	selection: null,

	  get rowCount()                     { return this.visibleData.length; },
	  setTree: function(treeBox)         { this.treeBox = treeBox; },
	  getCellText: function(idx, column) { return this.visibleData[idx][2]; },
	  isContainer: function(idx)         { return this.visibleData[idx][3]; },
	  isContainerOpen: function(idx)     { return this.visibleData[idx][4]; },
	  isContainerEmpty: function(idx)    { return false; },
	  isSeparator: function(idx)         { return false; },
	  isSorted: function()               { return false; },
	  isEditable: function(idx, column)  { return false; },
	
	  getParentIndex: function(idx) {
	    for (var t = idx - 1; t >= 0 ; t--) {
	      if (this.isContainer(t)) return t;
	    }
	    return -1;
	  },
	  
	  getLevel: function(idx) {
	  	return this.visibleData[idx][1];
	  },
	  
	  hasNextSibling: function(idx, after) {
	    var thisLevel = this.getLevel(idx);
	    for (var t = idx + 1; t < this.visibleData.length; t++) {
	      var nextLevel = this.getLevel(t)
	      if (nextLevel == thisLevel) return true;
	      else if (nextLevel < thisLevel) return false;
	    }
	  },
	  
	  toggleOpenState: function(idx) {
	    var item = this.visibleData[idx];
	    
	    if (!item[3]) return; //can't have children, probably shouldn't get here...
	
	    if (!item[4]) {	      
	      item[4] = true;
	      this.addChildren(item[0], idx + 1, item[1] + 1, 0, false, null);
	    } else {
		  item[4] = false;	
	      var thisLevel = this.getLevel(idx);
	      var deletecount = 0;
	      for (var t = idx + 1; t < this.visibleData.length; t++) {
	        if (this.getLevel(t) > thisLevel) deletecount++;
	        else break;
	      }
	      if (deletecount) {
	        this.visibleData.splice(idx + 1, deletecount);
	        this.treeBox.rowCountChanged(idx + 1, -deletecount);
	      }
	    }
	  },
	  
	  keySelectionChanged: function() {
	  	var item = this.visibleData[this.tree.currentIndex];
	  	var key;
	  	var uri = "sword://" + this.module.getInitials();
	  	if (item[0] == null) {
	  		//not a tree key
	  		key = "/" + escape(item[2]);
	  		//this.mini.loadURI(uri + key);
	  	} else {
	  		var cKey = item[0];
	  		key = "";
	  		while (cKey != null) {
	  			key = "/" + escape(cKey.getName()) + key;
	  			//key = escape(cKey.getName()) + "/" + key;
	  			cKey = cKey.getParent(); 
	  		}
	  		key = key.substring(1); //remove the first leading slash, even top level elements have a "root" parent which will lead to an extra / in the URI
	  		//fb.getGBrowser().loadURI(uri + key);
	  	}
	  	if (item[0] != null || this.jSword.utilKlass.isMap(this.module)) {
	  		fb.getGBrowser().loadURI(uri + key);
	  	} else {
	  		this.mini.loadURI(uri + key);	  	
	  		// must set favicon URI here inspite of it being embedded in page,
			// it does not register when content is loaded in sidebar browser, so
			// history / bookmarks would appear without favicon
	  		var favIconURI = this.jSword.utilKlass.getIconURIFor(this.module);
	  		var ios = Components.classes["@mozilla.org/network/io-service;1"]
	  		                            .getService(Components.interfaces.nsIIOService);
	  		var iconURI = ios.newURI(favIconURI, null, null);
	  		var pageURI = ios.newURI(uri + key, null, null);
	  		fb.getGBrowser().mFaviconService.setAndLoadFaviconForPage(pageURI, iconURI, true);
	  	}
	  },
	
	  getImageSrc: function(idx, column) {},
	  getProgressMode : function(idx,column) {},
	  getCellValue: function(idx, column) {},
	  cycleHeader: function(col, elem) {},
	  selectionChanged: function() {},
	  cycleCell: function(idx, column) {},
	  performAction: function(action) {},
	  performActionOnCell: function(action, index, column) {},
	  getRowProperties: function(idx, column, prop) {},
	  getCellProperties: function(idx, column, prop) {},
	  getColumnProperties: function(column, element, prop) {},

};